@@ -1,4 +1,4 @@
-# $Id: Request.pm 222 2005-09-17 18:04:07Z rcaputo $
+# $Id: Request.pm 249 2006-03-24 14:10:11Z rcaputo $
 
 package POE::Component::Client::HTTP::Request;
 use strict;
@@ -202,7 +202,14 @@ sub add_eof {
   # RFC 2616: "If a message is received with both a Transfer-Encoding
   # header field and a Content-Length header field, the latter MUST be
   # ignored."
+  #
+  # Google returns a Content-Length header with its HEAD request,
+  # generating "incomplete response" errors.  Added a special case to
+  # ignore content for HEAD requests.  This may thwart keep-alive,
+  # however.
+
   if (
+    $self->[REQ_REQUEST]->method() ne "HEAD" and
     defined $self->[REQ_RESPONSE]->content_length and
     not defined $self->[REQ_RESPONSE]->header("Transfer-Encoding") and
     $self->[REQ_OCTETS_GOT] < $self->[REQ_RESPONSE]->content_length
@@ -512,7 +519,7 @@ sub connect_error {
   my $host = $self->[REQ_HOST];
   my $port = $self->[REQ_PORT];
 
-  $message = "Cannon connect to $host:$port ($message)";
+  $message = "Cannot connect to $host:$port ($message)";
   $self->error (RC_INTERNAL_SERVER_ERROR, $message);
   return;
 }
